@@ -1,6 +1,6 @@
 package XAS::Factory;
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 use Badger::Factory::Class
   debug   => 0,
@@ -11,13 +11,12 @@ use Badger::Factory::Class
     alert       => 'XAS::Lib::Modules::Alerts',
     email       => 'XAS::Lib::Modules::Email',
     environment => 'XAS::Lib::Modules::Environment',
-    log         => 'XAS::Lib::Modules::Log',
-    logger      => 'XAS::Lib::Modules::Log',
-    locking     => 'XAS::Lib::Modules::Locking',
-    lockmgr     => 'XAS::Lib::Modules::Locking',
+    log         => 'XAS::Lib::Log',
+    logger      => 'XAS::Lib::Log',
+    lockmgr     => 'XAS::Lib::Lockmgr',
+    pidfile     => 'XAS::Lib::Pidfile',
     spool       => 'XAS::Lib::Modules::Spool',
     spooler     => 'XAS::Lib::Modules::Spool',
-    pidfile     => 'XAS::Lib::Modules::PidFile',
   }
 ;
 
@@ -71,15 +70,15 @@ This will load L<XAS::Lib::Modules::Environment|XAS::Lib::Modules::Environment>
 
 =item B<log logger>
 
-This will load L<XAS::Lib::Modules::Log|XAS::Lib::Modules::Log>
+This will load L<XAS::Lib::Log|XAS::Lib::Log>
 
 =item B<locking lockmgr>
 
-This will load L<XAS::Lib::Modules::Locking|XAS::Lib::Modules::Locking>.
+This will load L<XAS::Lib::Lockmgr|XAS::Lib::Lockmgr>.
 
 =item B<pidfile>
 
-This will load L<XAS::Lib::Modules::PidFile|XAS::Lib::Modules::PidFile>.
+This will load L<XAS::Lib::Pidfile|XAS::Lib::Pidfile>.
 
 =item B<spool spooler>
 
@@ -107,7 +106,7 @@ Kevin L. Esteb, E<lt>kevin@kesteb.usE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2014 Kevin L. Esteb
+Copyright (c) 2012-2015 Kevin L. Esteb
 
 This is free software; you can redistribute it and/or modify it under
 the terms of the Artistic License 2.0. For details, see the full text
