@@ -6,7 +6,6 @@ use Try::Tiny;
 use Pod::Usage;
 use Hash::Merge;
 use Getopt::Long;
-use POSIX 'setsid';
 use XAS::Lib::Pidfile;
 
 use XAS::Class
@@ -14,7 +13,7 @@ use XAS::Class
   version   => $VERSION,
   import    => 'class CLASS',
   base      => 'XAS::Lib::App',
-  utils     => ':process',
+  utils     => ':process dotid',
   constants => 'TRUE FALSE',
   accessors => 'daemon pid',
 ;
@@ -40,7 +39,7 @@ sub define_pidfile {
 
     $self->log->debug('entering define_pidfile()');
 
-    $self->{pid} = XAS::Lib::Pidfile->new(-pid => $$);
+    $self->{'pid'} = XAS::Lib::Pidfile->new(-pid => $$);
 
     if (my $num = $self->pid->is_running()) {
 
@@ -101,18 +100,18 @@ sub _default_options {
 
     my $options = $self->SUPER::_default_options();
 
-    $self->{daemon}  = FALSE;
+    $self->{'daemon'} = FALSE;
 
     $options->{'daemon'} = \$self->{daemon};
 
-    $options->{'cfgfile=s'} = sub { 
+    $options->{'cfg-file=s'} = sub { 
         my $cfgfile = File($_[1]);
-        $self->env->cfgfile($cfgfile);
+        $self->env->cfg_file($cfgfile);
     };
 
-    $options->{'pidfile=s'} = sub { 
+    $options->{'pid-file=s'} = sub { 
         my $pidfile = File($_[1]);
-        $self->env->pidfile($pidfile);
+        $self->env->pid_file($pidfile);
     };
 
     return $options;
@@ -152,7 +151,7 @@ documentation.
 =head2 define_pidfile
 
 This method sets up the pid file for the process. By default, this file
-is named $XAS_RUN/<$0>.pid. This can be overridden by the --pidfile option.
+is named $XAS_RUN/<$0>.pid. This can be overridden by the --pid-file option.
 
 =head2 define_signals
 
@@ -167,11 +166,11 @@ This method will cause the process to become a daemon.
 
 This module handles these additional options.
 
-=head2 --cfgfile
+=head2 --cfg-file
 
 This defines an optional configuration file.
 
-=head2 --pidfile
+=head2 --pid-file
 
 This defines the pid file for recording the pid.
 
@@ -183,6 +182,8 @@ Become a daemon.
 
 =over 4
 
+=item L<XAS::Lib::App|XAS::Lib::App>
+
 =item L<XAS|XAS>
 
 =back
@@ -193,7 +194,7 @@ Kevin L. Esteb, E<lt>kevin@kesteb.usE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2014 Kevin L. Esteb
+Copyright (c) 2012-2015 Kevin L. Esteb
 
 This is free software; you can redistribute it and/or modify it under
 the terms of the Artistic License 2.0. For details, see the full text
