@@ -1,12 +1,12 @@
 package XAS::Lib::Mixins::Handlers;
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 use XAS::Class
   debug   => 0,
   version => $VERSION,
   base    => 'XAS::Base',
-  utils   => 'compress',
+  utils   => ':validation compress',
   mixins  => 'exit_handler exception_handler error_handler parse_exception',
 ;
 
@@ -16,22 +16,15 @@ use XAS::Class
 
 sub exception_handler {
     my $self = shift;
+    my ($ex) = validate_params(\@_, [1]);
 
-    my ($ex) = $self->validate_params(\@_, [1]);
-
-    my $errors = $self->parse_exception($ex);
-    my $script = $self->class->any_var('SCRIPT');
+    my ($errors, $rc) = $self->parse_exception($ex);
 
     $self->log->error($errors);
 
-    if ($self->alerts->check) {
+    if ($self->env->alerts) {
 
-        $self->alert->send(
-            -process  => $script,
-            -priority => $self->priority,
-            -facility => $self->facility,
-            -message  => $errors
-        );
+        $self->alert->send($errors);
 
     }
 
@@ -39,21 +32,15 @@ sub exception_handler {
 
 sub exit_handler {
     my $self = shift;
+    my ($ex) = validate_params(\@_, [1]);
 
-    my ($ex) = $self->validate_params(\@_, [1]);
-    my $script = $self->class->any_var('SCRIPT');
     my ($errors, $rc) = $self->parse_exception($ex);
 
     $self->log->fatal($errors);
 
-    if ($self->alerts) {
+    if ($self->env->alerts) {
 
-        $self->alert->send(
-            -process  => $script,
-            -priority => $self->priority,
-            -facility => $self->facility,
-            -message  => $errors
-        );
+        $self->alert->send($errors);
 
     }
 
@@ -63,9 +50,9 @@ sub exit_handler {
 
 sub error_handler {
     my $self = shift;
+    my ($ex) = validate_params(\@_, [1]);
 
-    my ($ex) = $self->validate_params(\@_, [1]);
-    my $errors = $self->parse_exception($ex);
+    my ($errors, $rc) = $self->parse_exception($ex);
 
     $self->log->error($errors);
 
@@ -73,8 +60,7 @@ sub error_handler {
 
 sub parse_exception {
     my $self= shift;
-
-    my ($ex) = $self->validate_params(\@_, [1]);
+    my ($ex) = validate_params(\@_, [1]);
 
     my $rc = 0;
     my $errors;
@@ -82,9 +68,7 @@ sub parse_exception {
 
     if ($ref) {
 
-        if ($ex->can('info') && 
-            $ex->can('type') && 
-            $ex->can('match_type')) {
+        if ($ex->isa('Badger::Exception')) {
 
             my $type = $ex->type;
             my $info = compress($ex->info);
