@@ -4,60 +4,52 @@ our $VERSION = '0.03';
 
 use Try::Tiny;
 use XAS::Factory;
+use XAS::Constants 'LOCK_DRIVERS';
 
 use XAS::Class
   debug      => 0,
   version    => $VERSION,
   base       => 'XAS::Base',
   mixin      => 'XAS::Lib::Mixins::Handlers',
-  filesystem => 'File',
+  utils      => ':validation dotid',
+  filesystem => 'Dir File',
   accessors  => 'lockmgr',
-  utils      => 'dotid',
   vars => {
     PARAMS => {
       -directory => { isa => 'Badger::Filesystem::Directory' }, 
       -mask      => { optional => 1, default => 0664 },
+      -lock      => { optional => 1, default => undef },
       -extension => { optional => 1, default => '.pkt' },
       -seqfile   => { optional => 1, default => '.SEQ' },
-      -lockfile  => { optional => 1, default => 'spool' },
+      -driver    => { optional => 1, default => 'Filesystem', regex => LOCK_DRIVERS },
     }
   }
 ;
 
+#use Data::Dumper;
+
 # ------------------------------------------------------------------------
 # Public Methods
 # ------------------------------------------------------------------------
 
 sub read {
     my $self = shift;
-    my ($filename) = $self->validate_params(\@_, [
+    my ($filename) = validate_params(\@_, [
         { isa => 'Badger::Filesystem::File' },
     ]);
 
     my $packet;
 
-    if ($self->lockmgr->lock_directory($self->directory)) {
+    if ($self->lockmgr->lock($self->lock)) {
 
-        try {
-
-            $packet = $self->read_packet($filename);
-            $self->lockmgr->unlock_directory($self->directory);
-
-        } catch {
-
-            my $ex = $_;
-
-            $self->lockmgr->unlock_directory($self->directory);
-
-            die $ex;
-
-        };
+        $packet = $self->_read_packet($filename);
+        $self->lockmgr->unlock($self->lock);
 
     } else { 
 
         $self->throw_msg(
             dotid($self->class) . '.read.lock_error',
-            'spooler_lock_error', 
+            'lock_dir_error', 
             $self->directory->path
         );
 
@@ -69,34 +61,22 @@ sub read {
 
 sub write {
     my $self = shift;
-    my ($packet) = $self->validate_params(\@_, [ 1 ]);
+    my ($packet) = validate_params(\@_, [ 1 ]);
 
     my $seqnum;
 
-    if ($self->lockmgr->lock_directory($self->directory)) {
+    if ($self->lockmgr->lock($self->lock)) {
 
-        try {
+        $seqnum = $self->_sequence();
 
-            $seqnum = $self->sequence();
-
-            $self->write_packet($packet, $seqnum);
-            $self->lockmgr->unlock_directory($self->directory);
-
-        } catch {
-
-            my $ex = $_;
-
-            $self->lockmgr->unlock_directory($self->directory);
-
-            die $ex;
-
-        }
+        $self->_write_packet($packet, $seqnum);
+        $self->lockmgr->unlock($self->lock);
 
     } else { 
 
         $self->throw_msg(
             dotid($self->class) . '.write.lock_error', 
-            'lock_error', 
+            'lock_dir_error', 
             $self->directory->path
         ); 
 
@@ -110,18 +90,19 @@ sub scan {
     my $self = shift;
 
     my @files;
-    my $pattern = qr/$self->extension/i;
+    my $regex = $self->extension;
+    my $pattern = qr/$regex/i;
 
-    if ($self->lockmgr->lock_directory($self->directory)) {
+    if ($self->lockmgr->lock($self->lock)) {
 
-        @files = sort(grep( $_->path =~ /$pattern/, $self->directory->files() ));
-        $self->lockmgr->unlock_directory($self->directory);
+        @files = sort(grep( $_->path =~ $pattern, $self->directory->files() ));
+        $self->lockmgr->unlock($self->lock);
 
     } else {
 
         $self->throw_msg(
             dotid($self->class) . '.scan.lock_error',
-            'spooler_lock_error',
+            'lock_dir_error',
             $self->directory->path
         );
 
@@ -133,32 +114,20 @@ sub scan {
 
 sub delete {
     my $self = shift;
-    my ($file) = $self->validate_params(\@_, [
+    my ($file) = validate_params(\@_, [
         { isa => 'Badger::Filesystem::File' },
     ]);
 
-    if ($self->lockmgr->lock_directory($self->directory)) {
+    if ($self->lockmgr->lock($self->lock)) {
 
-        try {
-
-            $file->delete;
-            $self->lockmgr->unlock_directory($self->directory);
-
-        } catch {
-
-            my $ex = $_;
-
-            $self->lockmgr->unlock_directory($self->directory);
-
-            die $ex;
-
-        };
+        $file->delete;
+        $self->lockmgr->unlock($self->lock);
 
     } else { 
 
         $self->throw_msg(
             dotid($self->class) . '.delete.lock_error', 
-            'spooler_lock_error', 
+            'lock_dir_error', 
             $self->directory->path
         ); 
 
@@ -171,20 +140,21 @@ sub count {
 
     my @files;
     my $count;
-    my $pattern = qr/$self->extension/i;
+    my $regex = $self->extension;
+    my $pattern = qr/$regex/i;
 
-    if ($self->lockmgr->lock_directory($self->directory)) {
+    if ($self->lockmgr->lock($self->lock)) {
 
-        @files = grep( $_->path =~ /$pattern/, $self->directory->files() );
+        @files = grep( $_->path =~ $pattern, $self->directory->files() );
         $count = scalar(@files);
 
-        $self->lockmgr->unlock_directory($self->directory);
+        $self->lockmgr->unlock($self->lock);
 
     } else {
 
         $self->throw_msg(
             dotid($self->class) . '.count.lock_error',
-            'spooler_lock_error',
+            'lock_dir_error',
             $self->directory->path
         );
 
@@ -201,18 +171,18 @@ sub get {
     my $filename;
     my $pattern = qr/$self->extension/i;
 
-    if ($self->lockmgr->lock_directory($self->directory)) {
+    if ($self->lockmgr->lock($self->lock)) {
 
         @files = sort(grep( $_->path =~ /$pattern/, $self->directory->files() ));
         $filename = $files[0];
 
-        $self->lockmgr->unlock_directory($self->directory);
+        $self->lockmgr->unlock($self->lock);
 
     } else {
 
         $self->throw_msg(
             dotid($self->class) . '.get.lock_error',
-            'spooler_lock_error',
+            'lock_dir_error',
             $self->directory->path
         );
 
@@ -231,21 +201,47 @@ sub init {
 
     my $self = $class->SUPER::init(@_);
 
-    $self->{lockmgr} = XAS::Factory->module('lockmgr', {
-        -lockfile => $self->lockfile
-    });
-    
+    unless (defined($self->{'lock'})) {
+
+        $self->{'lock'} = Dir($self->env->locks, 'spool')->path;
+
+    }
+
+    $self->{'lockmgr'} = XAS::Factory->module('lockmgr');
+
+    $self->lockmgr->add(
+        -key    => $self->lock,
+        -driver => $self->driver,
+    );
+
     return $self;
 
 }
 
-sub sequence {
+sub _chmod {
+    my $self = shift;
+    my $file = shift;
+
+    my $mask = $self->mask + 0;
+    my $cnt = chmod($mask, $file);
+
+    if ($cnt < 1) {
+
+        $self->throw_msg(
+            dotid($self->class) . '.chmod.invperms', 
+            'invperms', 
+            $file
+        );
+
+    }
+
+}
+
+sub _sequence {
     my $self = shift;
 
     my $fh;
-    my $cnt;
     my $seqnum;
-    my $mask = $self->mask + 0;
     my $file = File($self->directory, $self->seqfile);
 
     try {
@@ -265,12 +261,7 @@ sub sequence {
             $fh->print("1");
             $fh->close;
 
-            $cnt = chmod($mask, $file);
-            $self->throw_msg(
-                dotid($self->class) . '.sequence.invperms', 
-                'invperms', 
-                $file
-            ) if ($cnt < 1);
+            $self->_chmod($file);
 
             $seqnum = 1;
 
@@ -292,13 +283,11 @@ sub sequence {
 
 }
 
-sub write_packet {
+sub _write_packet {
     my $self = shift;
-    my ($packet, $seqnum) = $self->validate_params(\@_, [1,1]);
+    my ($packet, $seqnum) = validate_params(\@_, [1,1]);
 
     my $fh;
-    my $cnt;
-    my $mask = $self->mask + 0;
     my $file = File($self->directory, $seqnum . $self->extension);
 
     try {
@@ -306,12 +295,8 @@ sub write_packet {
         $fh = $file->open("w");
         $fh->print($packet);
         $fh->close;
-        $cnt = chmod($mask, $file->path) or 
-          $self->throw_msg(
-              dotid($self->class) . '.write_packet.invperms',
-              'invperms',
-              $file->path
-          );
+
+        $self->_chmod($file->path);
 
     } catch {
 
@@ -327,9 +312,9 @@ sub write_packet {
 
 }
 
-sub read_packet {
+sub _read_packet {
     my $self = shift;
-    my ($file) = $self->validate_params(\@_, [
+    my ($file) = validate_params(\@_, [
         { isa => 'Badger::Filesystem::File' }
     ]);
 
@@ -372,7 +357,8 @@ XAS::Lib::Modules::Spool - A Perl extension for the XAS environment
 
  my $spl = XAS::Factory->module(
      spool => {
-         -directory => 'spool'
+         -directory => 'spool',
+         -lock      => 'spool',
      }
  );
 
@@ -419,9 +405,9 @@ This will initialize the base object. It takes the following parameters:
 
 This is the directory to use for spool files.
 
-=item B<-lockfile>
+=item B<-lock>
 
-The name of the lock file to use. Defaults to 'spool'.
+The name of the lock to use. Defaults to 'spool'.
 
 =item B<-extension>
 
@@ -483,9 +469,9 @@ This method will retrieve a file name from the spool directory.
 
 This method will get the current file extension.
 
-=head2 lockfile
+=head2 lock
 
-This method will get the current lock file name.
+This method will get the current locks name.
 
 =head2 segfile
 
