@@ -1,23 +1,30 @@
 package XAS::Lib::Stomp::Parser;
 
-our $VERSION = '0.01';
+our $VERSION = '0.03';
 
 use XAS::Lib::Stomp::Frame;
+use XAS::Constants ':stomp';
 
 use XAS::Class
-  debug   => 0,
-  version => $VERSION,
-  base    => 'XAS::Base',
-  mixin   => 'XAS::Lib::Mixins::Bufops',
-  utils   => 'trim',
+  debug     => 0,
+  version   => $VERSION,
+  base      => 'XAS::Base',
+  mixin     => 'XAS::Lib::Mixins::Bufops',
+  utils     => 'trim',
+  vars => {
+    PARAMS => {
+      -target  => { optional => 1, default => undef, regex => STOMP_LEVELS },
+    }
+  }
 ;
 
-our $EOF    = "\000";
-our $CNTRL  = qr((?:[[:cntrl:]])+);
-our $HEADER = qr(([\w\-~]+)\s*:\s*(.*));
-our $EOL    = qr((\015\012?|\012\015?|\015|\012));
-our $BEOH   = qr((\015\012\000?|\012\015\000?|\015\000|\012\000));
-our $EOH    = qr((\015\012\015\012?|\012\015\012\015?|\015\015|\012\012));
+
+my $HEADER = STOMP_HEADER;
+my $CNTRL  = STOMP_CNTRL;
+my $BEOH   = STOMP_BEOH;
+my $EOF    = STOMP_EOF;
+my $EOH    = STOMP_EOH;
+my $EOL    = STOMP_EOL;
 
 #use Data::Dumper;
 #use Data::Hexdumper;
@@ -40,7 +47,7 @@ sub parse {
 
     $self->{buffer} .= $buffer;
 
-    $self->log->debug('stomp parse');
+    $self->log->debug('stomp-parser: begin');
 #    $self->log->debug(hexdump($self->{buffer}));
 
     # A valid frame is usually this:
@@ -69,7 +76,7 @@ sub parse {
 
     for (;;) {
 
-        $self->log->debug('state = ' . $self->{state});
+        $self->log->debug('stomp-parser: state = ' . $self->{state});
 
         if ($self->{state} eq 'command') {
 
@@ -78,7 +85,7 @@ sub parse {
 
             if ($line = $self->buf_get_line(\$self->{buffer}, $EOL)) {
 
-                $self->log->debug('command');
+                $self->log->debug('stomp-parser: command');
 #                $self->log->debug(hexdump($line));
 
                 $line = trim($line);
@@ -93,14 +100,14 @@ sub parse {
             # start of the headers, they last until a standalone <eol>
             # or <eof> is reached. 
 
-            $self->log->debug("header");
+            $self->log->debug("stomp-parser: header");
 
             $length = length($self->{buffer});
 
             $self->{buffer} =~ m/$EOH/g;
             $clength = pos($self->{buffer}) || -1;
 
-            $self->log->debug("end of headers $clength");
+            $self->log->debug("stomp-parser: end of headers, length: $clength");
 
             if ($clength == -1) {
 
@@ -108,7 +115,7 @@ sub parse {
                 $self->{buffer} =~ m/$BEOH/g;
                 $clength = pos($self->{buffer}) || -1;
 
-                $self->log->debug("end of frame $clength");
+                $self->log->debug("stomp-parser: end of frame, length: $clength");
 
             }
 
@@ -119,7 +126,7 @@ sub parse {
 
                 while ($line =~ s/^$HEADER//) {
 
-                    $self->log->debug('valid header');
+                    $self->log->debug('stomp-parser: valid header');
 
                     my $key   = lc($1);
                     my $value = trim($2);
@@ -129,7 +136,7 @@ sub parse {
                     # the last defined header is honored. The duplictes
                     # are discarded.
 
-                    if ($self->env->mqlevel < 1.2) {
+                    if ($self->target < 1.2) {
 
                         $self->{headers}->{$key} = $value;
 
@@ -153,7 +160,7 @@ sub parse {
 
         } elsif ($self->{state} eq 'body') {
 
-            $self->log->debug('body');
+            $self->log->debug('stomp-parser: body');
 
             # start of the body, determine wither to use
             # content-length or EOF to find the end 
@@ -162,7 +169,7 @@ sub parse {
 
             if ($clength = $self->{headers}->{'content-length'}) {
 
-                $self->log->debug('using content-length');
+                $self->log->debug('stomp-parser: using content-length');
 
                 if ($clength <= $length) {
 
@@ -173,7 +180,7 @@ sub parse {
 
             } else {
 
-                $self->log->debug('using EOF');
+                $self->log->debug('stomp-parser: using EOF');
 
                 $clength = index($self->{buffer}, $EOF);
 
@@ -189,7 +196,7 @@ sub parse {
 
         } elsif ($self->{state} eq 'frame') {
 
-            $self->log->debug('building frame');
+            $self->log->debug('stomp-parser: building frame');
 
             # clear out inter-frame crap and create the object.
 
@@ -234,7 +241,13 @@ sub init {
     my $class = shift;
 
     my $self = $class->SUPER::init(@_);
-    
+
+    unless (defined($self->{target})) {
+
+        $self->{target} = $self->env->mqlevel;
+
+    }
+
     $self->{state} = 'command';
 
     return $self;
