@@ -14,27 +14,30 @@ use POSIX qw(:sys_wait_h setsid);
 use XAS::Class
   debug      => 0,
   version    => $VERSION,
-  base       => 'Badger::Utils XAS::Base',
-  constants  => 'HASH ARRAY',
+  base       => 'Badger::Utils',
+  constants  => 'HASH ARRAY LOG_LEVELS',
   filesystem => 'Dir File',
   exports => {
     all => 'db2dt dt2db trim ltrim rtrim daemonize hash_walk  
-            load_module bool init_module load_module compress exitcode 
-            kill_proc spawn _do_fork glob2regex dir_walk
-            env_store env_restore env_create env_parse env_dump
-            left right mid instr is_truthy is_falsey run_cmd',
+            load_module bool compress exitcode _do_fork glob2regex dir_walk
+            env_store env_restore env_create env_parse env_dump env_clear
+            left right mid instr is_truthy is_falsey run_cmd
+            validate_params validation_exception level2syslog
+            stat2text bash_escape create_argv de_camel_case',
     any => 'db2dt dt2db trim ltrim rtrim daemonize hash_walk  
-            load_module bool init_module load_module compress exitcode 
-            kill_proc spawn _do_fork glob2regex dir_walk
-            env_store env_restore env_create env_parse env_dump
-            left right mid instr is_truthy is_falsey run_cmd',
+            load_module bool compress exitcode _do_fork glob2regex dir_walk
+            env_store env_restore env_create env_parse env_dump env_clear
+            left right mid instr is_truthy is_falsey run_cmd
+            validate_params validation_exception level2syslog
+            stat2text bash_escape create_argv de_camel_case',
     tags => {
-      dates   => 'db2dt dt2db',
-      env     => 'env_store env_restore env_create env_parse env_dump',
-      modules => 'init_module load_module',
-      strings => 'trim ltrim rtrim compress left right mid instr',
-      process => 'daemonize spawn kill_proc exitcode _do_fork',
-      boolean => 'is_truthy is_falsey bool',
+      dates      => 'db2dt dt2db',
+      env        => 'env_store env_restore env_create env_parse env_dump env_clear',
+      modules    => 'load_module',
+      strings    => 'trim ltrim rtrim compress left right mid instr',
+      process    => 'daemonize exitcode run_cmd _do_fork bash_escape create_argv',
+      boolean    => 'is_truthy is_falsey bool',
+      validation => 'validate_params validation_exception',
     }
   }
 ;
@@ -45,9 +48,57 @@ use XAS::Class
 # Public Methods
 # ----------------------------------------------------------------------
 
+sub validation_exception {
+    my $param = shift;
+    my $class = shift;
+
+    my $format = 'invalid parameters passed, reason: %s';
+    my $method = Badger::Utils::dotid($class) . '.invparams';
+
+    $param = trim(lcfirst($param));
+
+    my $ex = XAS::Exception->new(
+        type => $method,
+        info => Badger::Utils::xprintf($format, $param),
+    );
+
+    $ex->throw();
+
+}
+
+sub validate_params {
+    my $params = shift;
+    my $specs  = shift;
+    my $class  = shift;
+
+    unless (defined($class)) {
+
+        $class = (caller(1))[3];
+
+    }
+
+    my $results = validate_with(
+        params => $params,
+        called => $class,
+        spec   => $specs,
+        normalize_keys => sub {
+            my $key = shift; 
+            $key =~ s/^-//; 
+            return lc $key;
+        },
+        on_fail => sub {
+            my $param = shift;
+            validation_exception($param, $class);
+        },
+    );
+
+    return wantarray ? @$results : $results;
+
+}
+
 # recursively walk a HOH
 sub hash_walk {
-    my $p = XAS::Base->validate_params(\@_, {
+    my $p = validate_params(\@_, {
         -hash     => { type => HASHREF }, 
         -keys     => { type => ARRAYREF }, 
         -callback => { type => CODEREF },
@@ -87,7 +138,7 @@ sub hash_walk {
 
 # recursively walk a directory structure
 sub dir_walk {
-    my $p = XAS::Base->validate_params(\@_, {
+    my $p = validate_params(\@_, {
         -directory => { isa  => 'Badger::Filesystem::Directory' },
         -callback  => { type => CODEREF },
         -filter    => { optional => 1, default => qr/.*/, callbacks => {
@@ -120,7 +171,7 @@ sub dir_walk {
 
 # Perl trim function to remove whitespace from the start and end of the string
 sub trim {
-    my ($string) = XAS::Base->validate_params(\@_, [1]);
+    my ($string) = validate_params(\@_, [1]);
 
     $string =~ s/^\s+//;
     $string =~ s/\s+$//;
@@ -131,7 +182,7 @@ sub trim {
 
 # Left trim function to remove leading whitespace
 sub ltrim {
-    my ($string) = XAS::Base->validate_params(\@_, [1]);
+    my ($string) = validate_params(\@_, [1]);
 
     $string =~ s/^\s+//;
 
@@ -141,7 +192,7 @@ sub ltrim {
 
 # Right trim function to remove trailing whitespace
 sub rtrim {
-    my ($string) = XAS::Base->validate_params(\@_, [1]);
+    my ($string) = validate_params(\@_, [1]);
 
     $string =~ s/\s+$//;
 
@@ -151,7 +202,7 @@ sub rtrim {
 
 # replace multiple whitspace with a single space
 sub compress {
-    my ($string) = XAS::Base->validate_params(\@_, [1]);
+    my ($string) = validate_params(\@_, [1]);
 
     $string =~ s/\s+/ /gms;
 
@@ -161,7 +212,7 @@ sub compress {
 
 # emulate Basics string function left()
 sub left {
-    my ($string, $offset) = XAS::Base->validate_params(\@_, [1,1]);
+    my ($string, $offset) = validate_params(\@_, [1,1]);
 
     return substr($string, 0, $offset);
 
@@ -169,7 +220,7 @@ sub left {
 
 # emulate Basics string function right()
 sub right {
-    my ($string, $offset) = XAS::Base->validate_params(\@_, [1,1]);
+    my ($string, $offset) = validate_params(\@_, [1,1]);
 
     return substr($string, -($offset));
 
@@ -177,7 +228,7 @@ sub right {
 
 # emulate Basics string function mid()
 sub mid {
-    my ($string, $start, $length) = XAS::Base->validate_params(\@_, [1,1,1]);
+    my ($string, $start, $length) = validate_params(\@_, [1,1,1]);
 
     return substr($string, $start - 1, $length);
 
@@ -185,7 +236,7 @@ sub mid {
 
 # emulate Basics string function instr()
 sub instr {
-    my ($start, $string, $compare) = XAS::Base->validate_params(\@_, [1,1,1]);
+    my ($start, $string, $compare) = validate_params(\@_, [1,1,1]);
 
     if ($start =~ /^[0-9\-]+/) {
 
@@ -203,11 +254,44 @@ sub instr {
 
 }
 
-# Checks to see if the parameter is the string 't', 'true', 'yes', '0E0' 
+sub de_camel_case {
+    my ($s) = validate_params(\@_, [1]);
+
+    my $o;
+    my @a = split('', $s);
+    my $z = scalar(@a);
+
+    for (my $x = 0; $x < $z; $x++) {
+
+        if ($a[$x] =~ /[A-Z]/) {
+
+            if ($x == 0) {
+
+                $o .= lc($a[$x]);
+
+            } else {
+
+                $o .= '_' . lc($a[$x]);
+
+            }
+
+        } else {
+
+            $o .= $a[$x];
+
+        }
+
+    }
+
+    return $o;
+
+}
+
+# Checks to see if the parameter is the string 't', 'true', 'yes', '0E0'
 # or the number 1.
 #
 sub is_truthy {
-    my ($parm) = XAS::Base->validate_params(\@_, [1]);
+    my ($parm) = validate_params(\@_, [1]);
 
     my @truth = qw(yes true t 1 0e0);
 
@@ -219,7 +303,7 @@ sub is_truthy {
 # the number 0.
 #
 sub is_falsey {
-    my ($parm) = XAS::Base->validate_params(\@_, [1]);
+    my ($parm) = validate_params(\@_, [1]);
 
     my @truth = qw(no false f 0);
 
@@ -228,134 +312,21 @@ sub is_falsey {
 }
 
 sub bool {
-    my ($item) = XAS::Base->validate_params(\@_, [1]);
+    my ($item) = validate_params(\@_, [1]);
 
     my @truth = qw(yes true 1 0e0 no false f 0);
     return grep {lc($item) eq $_} @truth;
 
 }
 
-sub spawn {
-    my $p = XAS::Base->validate_params(\@_, {
-        -command => 1,
-        -timeout => { optional => 1, default => 0 },
-    });
-
-    local $SIG{ALRM} = sub {
-        my $sig_name = shift;
-        die "$sig_name";
-    };
-
-    my $kid;
-    my @output;
-
-    defined( my $pid = open($kid, "-|" ) ) or do {
-
-        my $ex = XAS::Exception->new(
-            info => "unable to fork, reason: $!",
-            type => 'xas.utils.spawn'
-        );
-
-        $ex->throw;
-
-    };
-
-    if ($pid) {
-
-        # parent
-
-        try {
-
-            alarm( $p->{'timeout'} );
-
-            while (<$kid>) {
-
-                chomp;
-                push(@output, $_);
-
-            }
-
-            alarm(0);
-
-        } catch {
-
-            my $ex = $_;
-
-            alarm(0);
-
-            if ($ex =~ /alrm/i) {
-
-                unless (kill_proc(-signal => 'TERM', -pid => $pid)) {
-
-                    unless (kill_proc(-signal => 'KILL', -pid => $pid)) {
-
-                        my $ex = XAS::Exception->new(
-                            type => 'xas.utils.spawn',
-                            info => 'unable to kill ' . $pid
-                        );
-
-                        $ex->throw;
-
-                    }
-
-                }
-
-            } else {
-
-                die $ex;
-
-            }
-
-        };
-
-    } else {
-
-        # child
-
-        # set the child process to be a group leader, so that
-        # kill -9 will kill it and all its descendents
-
-        setpgrp(0, 0);
-        exec $p->{'command'};
-        exit;
-
-    }
-
-    wantarray ? @output : join( "\n", @output );
-
-}
-
-sub kill_proc {
-    my $p = XAS::Base->validate_params(\@_, {
-        -signal => 1,
-        -pid    => 1,
-    });
-
-    my $time = 10;
-    my $status = 0;
-    my $pid = $p->{'pid'};
-    my $signal = $p->{'signal'};
-
-    kill($signal, $pid);
-
-    do {
-
-        sleep 1;
-        $status = waitpid($pid, WNOHANG);
-        $time--;
-
-    } while ($time && not $status);
-
-    return $status;
-
-}
-
 sub exitcode {
 
-    my $rc  = $? >> 8;      # return code of command
-    my $sig = $? & 127;     # signal it was killed with
+    my $ex    = $?;
+    my $rc    = $ex >> 8;    # return code of process
+    my $sig   = $ex & 127;   # signal it was killed with
+    my $cored = $ex & 128;   # wither the process cored
 
-    return $rc, $sig;
+    return $rc, $sig, $cored;
 
 }
 
@@ -398,8 +369,8 @@ sub daemonize {
 }
 
 sub db2dt {
-    my ($p) = XAS::Base->validate_params(\@_, [
-        { regex => qr/\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}/ }
+    my ($p) = validate_params(\@_, [
+        { regex => qr/\d{4}-\d{2}-\d{2}.\d{2}:\d{2}:\d{2}/ }
     ]);
 
     my $parser = DateTime::Format::Strptime->new(
@@ -420,7 +391,7 @@ sub db2dt {
 }
 
 sub dt2db {
-    my ($dt) = XAS::Base->validate_params(\@_, [
+    my ($dt) = validate_params(\@_, [
         { isa => 'DateTime' }
     ]);
 
@@ -429,52 +400,17 @@ sub dt2db {
 }
 
 sub run_cmd {
-    my ($command) = XAS::Base->validate_params(\@_, [1]);
+    my ($command) = validate_params(\@_, [1]);
 
     my @output = `$command 2>&1`;
-    my ($rc, $sig) = exitcode();
+    my ($rc, $sig, $cored) = exitcode();
 
     return \@output, $rc, $sig;
 
 }
 
-sub init_module {
-    my ($module, $params) = XAS::Base->validate_params(@_, [ 
-        1, 
-        { optional => 1, type => HASHREF, default => {} }
-    ]);
-
-    my $obj;
-    my @parts;
-    my $filename;
-
-    @parts = split("::", $module);
-    $filename = File(@parts);
-
-    try {
-
-        require $filename . '.pm';
-        $module->import();
-        $obj = $module->new($params);
-
-    } catch {
-
-        my $x = $_;
-        my $ex = XAS::Exception->new(
-            type => 'xas.utils.init_module',
-            info => $x
-        );
-
-        $ex->throw;
-
-    };
-
-    return $obj;
-
-}
-
 sub load_module {
-    my ($module) = XAS::Base->validate_params(\@_, [1]);
+    my ($module) = validate_params(\@_, [1]);
 
     my @parts;
     my $filename;
@@ -502,7 +438,7 @@ sub load_module {
 }
 
 sub glob2regex {
-    my ($globstr) = XAS::Base->validate_params(\@_, [1]);
+    my ($globstr) = validate_params(\@_, [1]);
 
     my %patmap = (
         '*' => '.*',
@@ -517,42 +453,205 @@ sub glob2regex {
 
 }
 
-sub env_store {
+sub stat2text {
+    my ($stat) = validate_params(\@_, [1]);
+    
+    my $status = 'unknown';
 
-    my %env;
+    $status = 'suspended ready'   if ($stat == 6);
+    $status = 'suspended blocked' if ($stat == 5);
+    $status = 'blocked'           if ($stat == 4);
+    $status = 'running'           if ($stat == 3);
+    $status = 'ready'             if ($stat == 2);
+    $status = 'other'             if ($stat == 1);
 
-    while ((my $key, my $value) = each(%ENV)) {
-
-        delete $ENV{$key};
-        $env{$key} = $value;
-
-    }
-
-    return \%env;
+    return $status;
 
 }
 
-sub env_restore {
-    my ($env) = XAS::Base->validate_params(\@_, [1]);
+sub level2syslog {
+    my ($level) = validate_params(\@_, [
+        { regex => LOG_LEVELS },
+    ]);
 
-    while ((my $key, my $value) = each(%ENV)) {
+    my $translate = {
+        info  => 'info',
+        error => 'err',
+        warn  => 'warning',
+        fatal => 'alert',
+        trace => 'notice',
+        debug => 'debug'
+    };
 
+    return $translate->{lc($level)};
+
+}
+
+# ********************************************************************** 
+# The Bourne shell treats some characters in a command's argument list as
+# having a special meaning.  This could result in the shell executing    
+# unwanted commands. This code escapes the special characters by         
+# prefixing them with the \ character.                                   
+#
+# taken from: https://www.slac.stanford.edu/slac/www/resource/how-to-use/cgi-rexx/cgi-esc.html
+#
+# **********************************************************************
+
+sub bash_escape {
+    my $arg = shift;
+
+    $arg =~ s/([;<>\*\|&\$!#\(\)\[\]\{\}:'"])/\\$1/g;
+
+    return $arg;
+
+}
+
+#
+# Extracted form Parse::CommandLine.
+#
+
+sub create_argv {
+    my ($str) = validate_params(\@_, [1]);
+
+    $str =~ s/\A\s+//ms;
+    $str =~ s/\s+\z//ms;
+
+    my @argv;
+    my $buf;
+    my $escaped;
+    my $double_quoted;
+    my $single_quoted;
+
+    for my $char (split //, $str) {
+
+        if ($escaped) {
+
+            $buf .= $char;
+            $escaped = undef;
+            next;
+
+        }
+
+        if ($char eq '\\') {
+
+            if ($single_quoted) {
+
+                $buf .= $char;
+
+            } else {
+
+                $escaped = 1;
+
+            }
+            next;
+
+        }
+
+        if ($char =~ /\s/) {
+
+            if ($single_quoted || $double_quoted) {
+
+                $buf .= $char;
+
+            } else {
+
+                push @argv, $buf if defined $buf;
+                undef $buf;
+
+            }
+            next;
+
+        }
+
+        if ($char eq '"') {
+
+            if ($single_quoted) {
+
+                $buf .= $char;
+                next;
+
+            }
+
+            $double_quoted = !$double_quoted;
+            next;
+
+        }
+
+        if ($char eq "'") {
+
+            if ($double_quoted) {
+
+                $buf .= $char;
+                next;
+
+            }
+
+            $single_quoted = !$single_quoted;
+            next;
+
+        }
+
+        $buf .= $char;
+
+    }
+
+    push @argv, $buf if defined $buf;
+
+    if ($escaped || $single_quoted || $double_quoted) {
+
+        my $ex = XAS::Exception->new(
+            type => 'xas.utils.create_argv',
+            info => 'invalid command line string',
+        );
+
+        $ex->throw;
+
+    }
+
+    return @argv;
+
+}
+
+sub env_store {
+
+    my $env;
+
+    while (my ($key, $value) = each(%ENV)) {
+
+        $env->{$key} = $value;
+
+    }
+
+    return $env;
+
+}
+
+sub env_clear {
+    
+    while (my ($key, $value) = each(%ENV)) {
+        
         delete $ENV{$key};
-
+        
     }
+    
+}
 
-    while ((my $key, my $value) = each(%{$env})) {
+sub env_restore {
+    my ($env) = validate_params(\@_, [
+        { type => HASHREF },
+    ]);
 
-        $ENV{$key} = $value;
-
-    }
+    env_clear();
+    env_create($env);
 
 }
 
 sub env_create {
-    my ($env) = XAS::Base->validate_params(\@_, [1]);
+    my ($env) = validate_params(\@_, [
+        { type => HASHREF },
+    ]);
 
-    while ((my $key, my $value) = each(%{$env})) {
+    while (my ($key, $value) = each(%$env)) {
 
         $ENV{$key} = $value;
 
@@ -561,19 +660,21 @@ sub env_create {
 }
 
 sub env_parse {
-    my ($env) = XAS::Base->validate_params(\@_, [1]);
+    my ($e) = validate_params(\@_, [
+        { type => SCALAR },
+    ]);
 
-    my ($key, $value, %env);
-    my @envs = split(';;', $env);
+    my $env;
+    my @envs = split(';;', $e);
 
     foreach my $y (@envs) {
 
-        ($key, $value) = split('=', $y);
-        $env{$key} = $value;
+        my ($key, $value) = split('=', $y);
+        $env->{$key} = $value;
 
     }
 
-    return \%env;
+    return $env;
 
 }
 
@@ -581,7 +682,7 @@ sub env_dump {
 
     my $env;
 
-    while ((my $key, my $value) = each(%ENV)) {
+    while (my ($key, $value) = each(%ENV)) {
 
         $env .= "$key=$value;;";
 
@@ -621,15 +722,63 @@ namespace.
 
 =head1 METHODS
 
+=head2 validate_params($params, $spec, $class)
+
+This method is used to validate parameters. Internally this uses 
+Params::Validate::validate_with() for the parameter validation. 
+
+By convention, all named parameters have a leading dash. This method will 
+strip off that dash and lower case the parameters name.
+
+If an validation exception is thrown, the parameter name will have the dash 
+stripped.
+
+Based on the $spec, this can return an array or a hashref of validated
+parameters and values. 
+
+=over 4
+
+=item B<$params>
+
+An array ref to a set of parameters. 
+
+=item B<$spec>
+
+A validation spec as defined by L<Params::Validate|https://metacpan.org/pod/Params::Validate>.
+
+=item B<$class>
+
+An optional class that is calling this method. If one is not provided then
+caller() is used to determine the calling method.
+
+=back
+
+=head2 validation_exception($param, $class)
+
+This is a package level sub routine. It exists to provide a uniform exception
+error message. It takes these parameters:
+
+=over 4
+
+=item B<$param>
+
+The error message returned by L<Params::Validate|https://metacpan.org/pod/Params::Validate>.
+
+=item B<$class>
+
+The routine that the error occurred in.
+
+=back
+
 =head2 db2dt($datestring)
 
 This routine will take a date format of YYYY-MM-DD HH:MM:SS and convert it
-into a DateTime object.
+into a L<DateTime|https://metacpan.org/pod/DateTime> object.
 
 =head2 dt2db($datetime)
 
-This routine will take a DateTime object and convert it into the following
-string: YYYY-MM-DD HH:MM:SS
+This routine will take a L<DateTime|https://metacpan.org/pod/DateTime>
+object and convert it into the following string: YYYY-MM-DD HH:MM:SS
 
 =head2 trim($string)
 
@@ -671,39 +820,37 @@ Return the position in $string of $compare. You may offset within the
 string with $start. Useful for porting VBS code. Makes allowances that
 VBS strings are one based while Perls are zero based.
 
-=head2 spawn
+=head2 de_camel_case($string)
 
-Run a cli command with timeout. Returns output from that command.
-
-=over 4
-
-=item B<-command>
-
-The command string to run.
-
-=item B<-timeout>
-
-An optional timeout in seconds. Default is none.
-
-=back
+Break up a "CamelCase" string into a "camel_case" string. The opposit of
+camel_case() from L<Badger::Utils|https://metacpan.org/pod/Badger::Utils>.
 
 =head2 exitcode
 
-Decodes Perls version of the exit code from a cli process. Returns two items.
+Decodes Perls version of the exit code from a cli process. Returns three items.
 
  Example:
 
-     my @output = spawn(-command => "ls -l");
-     my ($rc, $sig) = exitcode();
+     my @output = `ls -l`;
+     my ($rc, $sig, $cored) = exitcode();
 
 =head2 run_cmd($command)
 
-Run a command and capture the output and exit code, stderr is merged
-with stdout.
+Run a command and capture the output, exit code and exit signal, stderr 
+is merged with stdout.
 
  Example:
  
-     my (@output, $rc) = run_cmd("ls -l");
+     my ($output, $rc, $sig) = run_cmd("ls -l");
+     if ($rc == 0) {
+
+         foreach my $line (@$output) {
+
+             print $line;
+
+         }
+
+     }
 
 =head2 daemonize
 
@@ -781,23 +928,6 @@ The callback to execute when matching files are found.
 
 =back
 
-=head2 init_module($module, $options)
-
-This routine will load and initialize a module. It takes one required parameter
-and one optinal parameter.
-
-=over 4
-
-=item B<$module>
-
-The name of the module.
-
-=item B<$options>
-
-A hashref of optional options to use with the module.
-
-=back
-
 =head2 load_module($module)
 
 This routine will load a module. 
@@ -810,6 +940,45 @@ The name of the module.
 
 =back
 
+=head2 stat2text($stat)
+
+This will convert the numeric process status to a text string.
+
+=over 4
+
+=item B<$stat>
+
+A number between 0 and 6.
+
+ 0 = 'unknown'
+ 1 = 'other'
+ 2 = 'ready'
+ 3 = 'running'
+ 4 = 'blocked'
+ 5 = 'suspended blocked'
+ 6 = 'suspended ready'
+
+=back
+
+=head2 level2syslog($level)
+
+This will convert a XAS log level to an appropriate syslog priority.
+
+=over 4
+
+=item B<$level>
+
+A XAS log level, it should be lower cased.
+
+ info  = 'info',
+ error = 'err',
+ warn  = 'warning',
+ fatal = 'alert',
+ trace = 'notice',
+ debug = 'debug'
+
+=back
+
 =head2 env_store
 
 Remove all items from the $ENV variable and store them in a hash variable.
@@ -833,7 +1002,7 @@ Store all the items from a hash variable into the $ENV varable.
 
 =head2 env_parse
 
-Take a formated string and parse it into a hash variable. The string must have
+Take a formatted string and parse it into a hash variable. The string must have
 this format: "item=value;;item2=value2";
 
   Example:
@@ -843,7 +1012,7 @@ this format: "item=value;;item2=value2";
 
 =head2 env_dump
 
-Take the items from the current $ENV variable and create a formated string.
+Take the items from the current $ENV variable and create a formatted string.
 
   Example:
     my $string = env_dump();
@@ -865,7 +1034,7 @@ Kevin L. Esteb, E<lt>kevin@kesteb.usE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2014 Kevin L. Esteb
+Copyright (c) 2012-2015 Kevin L. Esteb
 
 This is free software; you can redistribute it and/or modify it under
 the terms of the Artistic License 2.0. For details, see the full text
